//
// Vrml.java
//

import java.io.*;

public class Vrml
{
    // constructor
    public Vrml(String file, String name) throws IOException
    {
	outFile = new PrintWriter (new FileWriter (file) );
        myName = name;
    }

    // method 1 -- VrmlTruss
    // Draws the function defined in ftheta on the canvas
    public void VrmlTruss(double node1[], double node2[], double force) throws IOException
    {
	outFile.print("# Draw truss member\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\ttranslation 0 0 0");
	outFile.print("\tchildren Shape {\n");
	outFile.print("\t\tappearance Appearance {\n");
	outFile.print("\t\t\tmaterial Material {\n");
	if (force < -EPS)       // in tension
		outFile.print("\t\t\t\tdiffuseColor 1 0 0\n\t\t\t}\n\t\t}\n");
	else if (force > EPS)   // in compression
		outFile.print("\t\t\t\tdiffuseColor 0 0 1\n\t\t\t}\n\t\t}\n");
	else                    // neutral
		outFile.print("\t\t\t\tdiffuseColor 1 0 1\n\t\t\t}\n\t\t}\n");
	outFile.print("\t\tgeometry Extrusion {\n");
	outFile.print("\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n");
	outFile.print("\t\t\tscale [");
        outFile.print(TRUSS_WIDTH);
        outFile.print(" ");
        outFile.print(TRUSS_WIDTH);
        outFile.print("]\n");
	outFile.print("\t\t\tspine [");
	outFile.print(node1[0]);
        outFile.print(" ");
        outFile.print(node1[1]);
        outFile.print(" ");
        outFile.print(node1[2]);
        outFile.print(", ");
        outFile.print(node2[0]);
        outFile.print(" ");
        outFile.print(node2[1]);
        outFile.print(" ");
        outFile.print(node2[2]);
        outFile.print("]\n");
	outFile.print("\t\t}\n\t}\n}\n");

	outFile.print("# Draw force string\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\ttranslation ");
        outFile.print(0.5*(node1[0]+node2[0]));
        outFile.print(" ");
        outFile.print(0.5*(node1[1]+node2[1])-0.3);
        outFile.print(" ");
	outFile.print(0.5*(node1[2]+node2[2])+0.3);
        outFile.print("\n");
	outFile.print("\tchildren Shape {\n");
	outFile.print("\t\tappearance Appearance {\n");
	outFile.print("\t\t\tmaterial Material {\n");
	outFile.print("\t\t\t\tdiffuseColor 0 0.5 0\n");
	outFile.print("\t\t\t}\n\t\t}\n");
	outFile.print("\t\tgeometry Text {\n");
	outFile.print("\t\t\tstring [\"");
        outFile.print(force);
        outFile.print(" kN\"]\n");
	outFile.print("\t\t\t\tfontStyle FontStyle {\n");
	outFile.print("\t\t\t\tfamily \"SANS\"\n");
	outFile.print("\t\t\t\tjustify \"MIDDLE\"\n");
	outFile.print("\t\t\t\tstyle \"BOLD\"\n");
	outFile.print("\t\t\t\tsize 0.6\n\t\t\t}\n\t\t}\n\t}\n}\n\n");
    }

    // method 2 -- VrmlAxis
    // Draw in the output vrml file, canvas, background, heading, and axis
    public void VrmlAxis() throws IOException
    {
	// Create Header
	outFile.print("#VRML V2.0 utf8\n");
	outFile.print("#------------------------------------------------\n");
	outFile.print("# 24-311: Numerical Methods \n");
	outFile.print("# ");
        outFile.print(myName);
        outFile.print("\n");
	outFile.print("# Gauss Jordan with Partial Pivoting Output\n");
	outFile.print("#------------------------------------------------\n\n");

	// Set up background
	outFile.print("# Specify the background color\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Background {\n\tskyColor 1 1 1\n}\n\n"); // White background

	// Put text heading on the VRML screen
	outFile.print("# Draw texts\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\ttranslation 0 8 0\n");
	outFile.print("\tchildren Shape {\n");
	outFile.print("\t\tappearance Appearance {\n");
	outFile.print("\t\t\tmaterial Material {\n");
	outFile.print("\t\t\t\tdiffuseColor 0 0 0\n");
	outFile.print("\t\t\t}\n\t\t}\n");
	outFile.print("\t\tgeometry Text {\n");
	outFile.print("\t\t\tstring [\"24-311: Numerical Methods\",\n");
	outFile.print("\t\t\t\"Gauss Jordan with Partial Pivoting\",\n");
	outFile.print("\t\t\t\"");
        outFile.print(myName);
        outFile.print("\"]\n");
	outFile.print("\t\t\tfontStyle FontStyle {\n");
	outFile.print("\t\t\t\tfamily \"SANS\"\n");
	outFile.print("\t\t\t\tjustify \"MIDDLE\"\n");
	outFile.print("\t\t\t\tstyle \"BOLD\"\n");
	outFile.print("\t\t\t\tsize 0.6\n\t\t\t}\n\t\t}\n\t}\n}\n\n");

	// Add the x-axis
	outFile.print("# Define the x axis\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\ttranslation 0 0 0\n");
	outFile.print("\tchildren DEF AXIS Shape {\n");
	outFile.print("\t\tappearance Appearance {\n");
	outFile.print("\t\t\tmaterial Material {\n");
	outFile.print("\t\t\t\tdiffuseColor 0.0 0.0 0.0\n\t\t\t}\n\t\t}\n");
	outFile.print("\t\tgeometry Extrusion {\n");
	outFile.print("\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n");
	outFile.print("\t\t\tscale [0.035 0.035, 0.035 0.035, 0.15 0.15, 0 0]\n");
	outFile.print("\t\t\tspine [0 0 0, 6 0 0, 6 0 0, 6.5 0 0]\n\t\t}\n\t}\n}\n\n");

	// Add the y-axis
	outFile.print("# Draw the y axis\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\trotation 0 0 1 1.5707\n");
	outFile.print("\tchildren USE AXIS\n}\n\n");

	// Add the z-axis
	outFile.print("# Draw the z axis\n");
	outFile.print("#-----------------------------------\n");
	outFile.print("Transform {\n");
	outFile.print("\trotation 0 1 0 -1.5707\n");
	outFile.print("\tchildren USE AXIS\n}\n\n");
    }

    // method 3 -- VrmlClose : call this last
    public void VrmlClose() throws IOException
    {
	outFile.close();
    }

    // class variables
    private PrintWriter outFile;
    private String myName;

    // Constants
    private final double EPS = 0.01;
    private final double TRUSS_WIDTH = 0.1;
}
